------------------------------------------------------------------------------
--  Thin Ada95 binding to OCI (Oracle Call Interface)                    --
--  Copyright (C) 2000-2002 Dmitriy Anisimkov.                              --
--  License agreement and authors contact information are in file oci.ads   --
------------------------------------------------------------------------------
--  Oracle Call Interfase is Trademark of Oracle Corporation              --
------------------------------------------------------------------------------

--  $Id$

with Ada.Unchecked_Deallocation;
package body OCI.Reference_Finalization is

   procedure Free is new Ada.Unchecked_Deallocation (Counter, Access_Count);
   
   protected body Counter is
   
      procedure Dec (Zero : out Boolean) is
      begin
         Zero := Data = 0;
         Data := Data - 1;
      end;
      
      procedure Inc is
      begin
        Data := Data + 1;
      end;
      
   end Counter;

   procedure Initialize (Object : in out Controlled_Reference) is
   begin
      Object.Refer := new Counter;
   end;
   
   ------------
   -- Adjust --
   ------------

   procedure Adjust (Object : in out Controlled_Reference) is
   begin
      Object.Refer.Inc;
   end Adjust;

   --------------
   -- Finalize --
   --------------

   procedure Finalize (Object : in out Controlled_Reference) is
      Zero : Boolean;
   begin
      Object.Refer.Dec (Zero);
      if Zero then
         Destroy (Controlled_Reference'Class (Object));
         Free (Object.Refer);
      end if;
   end Finalize;

end OCI.Reference_Finalization;

