------------------------------------------------------------------------------
--  Thin Ada95 binding to OCI (Oracle Call Interface)                    --
--  Copyright (C) 2000-2002 Dmitriy Anisimkov.                              --
--  License agreement and authors contact information are in file oci.ads   --
------------------------------------------------------------------------------

--  $Id$

with OCI.Thread;
with Ada.Unchecked_Deallocation;

package body OCI.Thick.Binds is

   ----------
   -- Bind --
   ----------
   
   procedure Bind (
       Stmt : Statement;
       Var : in out Abstract_Bind'Class;
       Name : String;
       Type_Id : Ub2; Value : DVoid; Size : Sb4)
    is
       rc : SWord := OCIBindByName (
                stmtp => OCIStmt (Handle (Stmt)),
                bindpp => Var.Bind'Access,
                errhp => Thread.Error,
                placeholder => C.To_C(Name),
                placeh_len => Name'Length,
                valuep => Value,
                value_sz => Size,
                dty => Type_Id,
                indp => Var.Indicator);
   begin
--     Put_Line (Address_Image (Value));
     Check_Error(rc);
     Var.Stmt := Stmt;
   end;
    
   -----------------
   -- Clear_Value --
   -----------------

   procedure Clear_Value (Var : in out Abstract_Bind'Class) is
   begin
      Var.Indicator.all := Null_Indicator;
   end Clear_Value;

   -------------
   -- Is_Null --
   -------------

   function Is_Null (Var : Abstract_Bind'Class) return Boolean is
      use type Sb2;
   begin
      if Var.Bind = OCIBind (Empty_Handle) then
         raise NOT_ATTACHED;
      end if;
      return Var.Indicator.all = Null_Indicator;
   end;

   procedure Destroy (Object : in out Abstract_Bind)
   is
      procedure Free is new Ada.Unchecked_Deallocation (Sb2, A_Sb2);
   begin
      Free (Object.Indicator);
   end Destroy;

end OCI.Thick.Binds;

