with OCI.Lib;
with OCI.Thread;
with Interfaces.C;
with System.Address_To_Access_Conversions;
with Ada.Exceptions;
with Ada.Text_IO; use Ada.Text_IO; 
with System.Address_Image;

package body OCI.Thick.Notifications is
   
   use type System.Address;
   use OCI.Lib;

   To_UB4 : array (Namespace_Enum) of UB4 := 
      (AQ => OCI_SUBSCR_NAMESPACE_AQ,
       Anonymous => OCI_SUBSCR_NAMESPACE_Anonymous);

   procedure Notify
     (Item   : in out Subscription;
      Buffer : in     Buffer_Type)
   is
   begin
      null;
   end;

   --------------
   -- Finalize --
   --------------

   procedure Finalize (Object : in out Subscription) is
   begin
      if Object.Handle /= OCISubscription (Empty_Handle) then
         Free (H => OCIHandle (Object.Handle),
            HType => OCI_HTYPE_SUBSCRIPTION);
      end if;
   end Finalize;
   
   function OCISubscriptionNotify (
       pCtx : DVoid;
       pSubscrHp : OCISubscription;
       pPayload : DVoid;
       iPayloadLen : Ub4;
       pDescriptor : DVoid;
       iMode : Ub4) return SWord;
       
   pragma Convention (C, OCISubscriptionNotify);
   
   function OCISubscriptionNotify (
       pCtx : DVoid;
       pSubscrHp : OCISubscription;
       pPayload : DVoid;
       iPayloadLen : Ub4;
       pDescriptor : DVoid;
       iMode : Ub4) return SWord
   is
      package Converter is new
         System.Address_To_Access_Conversions (Subscription'Class);

      use Converter;
      use Ada.Text_IO;
      Subscr : Object_Pointer := To_Pointer (pCtx);
--      Queue_Name : String := Get_Attr (pDescriptor,
--         OCI_DTYPE_AQNFY_DESCRIPTOR, OCI_ATTR_QUEUE_NAME);
--      CONSUMER_Name : String := Get_Attr (pDescriptor,
--         OCI_DTYPE_AQNFY_DESCRIPTOR, OCI_ATTR_CONSUMER_NAME);
      Buffer : Buffer_Type := (Ptr => pPayload, Length => Natural (iPayloadLen));
   begin
      Notify (Subscr.all, Buffer);
      return OCI_CONTINUE;
   exception
      when E : others =>
         Put_Line (Current_Error, Ada.Exceptions.Exception_Information (E));
      return OCI_ERROR;
   end OCISubscriptionNotify;

   procedure Create (
      Item : in out Subscription;
      Name : String;
      Namespace : Namespace_Enum)
   is
   begin
      if Item.Handle /= OCISubscription (Empty_Handle) then
         -- Avoid memory and handles leak.
         raise Already_Registered;
      end if;

      Item.Handle := OCISubscription (Alloc_Handle (Thread.Environment, OCI_HTYPE_SUBSCRIPTION));

      Set_Attr ( OCIHandle (Item.Handle),
         OCI_HTYPE_SUBSCRIPTION,
         Name,
         OCI_ATTR_SUBSCR_NAME);

      Set_Attr (OCIHandle (Item.Handle), 
         OCI_HTYPE_SUBSCRIPTION,
         To_UB4 (Namespace)'Address,
         OCI_ATTR_SUBSCR_NAMESPACE);


   end Create;
      

   --------------
   -- Register --
   --------------

   procedure Register
     (Item : in out Subscription;
      Connect : Connection)
   is
      RC : SWord;
      Connect_Handle : OCIHandle := Handle (Connect);
   begin

      Set_Attr (OCIHandle (Item.Handle),
         OCI_HTYPE_SUBSCRIPTION,
         OCISubscriptionNotify'Address,
         OCI_ATTR_SUBSCR_CALLBACK);

      Set_Attr (OCIHandle (Item.Handle),
         OCI_HTYPE_SUBSCRIPTION,
         Item'Address,
         OCI_ATTR_SUBSCR_CTX);

      RC := OCISubscriptionRegister (
             svchp => OCISvcCtx (Connect_Handle),
             subscrhpp  => Item.Handle'Access,
             count => 1,
             errhp => Thread.Error,
             mode  => OCI_DEFAULT);

      Check_Error (RC);
      Item.Connect := Connect;
--   exception
--      when LIB_ERROR =>
--        Free (OCIHandle (Item.Handle), OCI_HTYPE_SUBSCRIPTION);
--        raise;
   end Register;

   procedure Unregister (
      Item : in out Subscription)
   is
      RC : SWord := OCISubscriptionUnRegister (
         svchp => OCISvcCtx (Handle (Item.Connect)),
         subscrhp => Item.Handle,
         errhp => Thread.Error,
         mode => OCI_DEFAULT);
   begin
      Check_Error (RC);
   end Unregister;

   ---------------
   -- To_String --
   ---------------

   function To_String (Buffer : in Buffer_Type) return String is
      use Interfaces.C;
      type Data_Type is new Char_Array (0 .. size_t (Buffer.Length) - 1);
      package Converter is new System.Address_To_Access_Conversions (Data_Type);
      use Converter;
      Ptr : Object_Pointer := To_Pointer (Buffer.Ptr);
   begin
      if Ptr = null then
         return "";
      else
         return To_Ada
           (Ptr.all,
            Trim_Nul  => False);
      end if;
   end To_String;

   procedure Post (
      Item    : in out Subscription;
      Connect :        Connection;
      Data : String)
   is
      RC : SWord;
      Buffer : C.Char_Array := C.To_C (Data);
   begin
      RC := OCIAttrSet (Trgthndlp => OCIHandle (Item.Handle),
         Trghndltyp => OCI_HTYPE_SUBSCRIPTION,
         Attributep => Buffer,
         Size => Data'Length,
         Attrtype => OCI_ATTR_SUBSCR_PAYLOAD,
         Errhp => Thread.Error);
      Check_Error (RC);
      RC := OCISubscriptionPost (
             svchp => OCISvcCtx (Handle (Connect)),
             subscrhpp  => Item.Handle'Access,
             count => 1,
             errhp => Thread.Error,
             mode  => OCI_DEFAULT);
      Item.Connect := Connect;
      Check_Error (RC);
      RC := OCIAttrSet (Trgthndlp => OCIHandle (Item.Handle),
         Trghndltyp => OCI_HTYPE_SUBSCRIPTION,
         Attributep => System.Null_Address,
         Size => 0,
         Attrtype => OCI_ATTR_SUBSCR_PAYLOAD,
         Errhp => Thread.Error);
      Check_Error (RC);
   end;

   procedure Enable (
      Item : in Subscription)
   is
      RC : SWord := OCISubscriptionEnable (
         Item.Handle, Thread.Error, OCI_DEFAULT);
   begin
      Check_Error (RC);
   end;
   
   procedure Disable (
      Item : in Subscription)
   is
      RC : SWord := OCISubscriptionDisable (
         Item.Handle, Thread.Error, OCI_DEFAULT);
   begin
      Check_Error (RC);
   end;

end OCI.Thick.Notifications;

