------------------------------------------------------------------------------
--  Thin Ada95 binding to OCI (Oracle Call Interface)                    --
--  Copyright (C) 2000-2002 Dmitriy Anisimkov.                              --
--  License agreement and authors contact information are in file oci.ads   --
------------------------------------------------------------------------------

--  $Id$

with
   Ada.Finalization,
   Ada.Task_Identification,
   System,
   Oci.Lib,
   OCI.Reference_Finalization;

--   uncomment it for debug printing
--   with Text_IO, System.Address_Image; use Text_IO,System;

package OCI.Thick is
   
   package RF renames Reference_Finalization;

   use Lib; 
   use Ada.Task_Identification;

   type Limited_Variable is tagged limited private;
   
   subtype OCINumber is LIB.OCINumber;
   
   procedure Clear_Value (Var : in out Limited_Variable'Class);
   function Is_Null (Var : Limited_Variable'Class) return Boolean;
   
   INVALID_HANDLE : exception;
   LIB_ERROR      : exception;
   NULL_VALUE     : exception;
   NOT_ATTACHED   : exception; -- trying get not attached variable

   function Last_Error_Code return Integer;
   
   function Last_Error_Message (Record_No : Integer := 1) return String;
   
   -- Position of last error in the DML statement.
   function Last_Error_Offset return Natural;

   type Handle_Reference is abstract new Rf.Controlled_Reference with private;
   
   procedure Check_Error (Code : Sword; Raise_Exception : Boolean := True);

   Too_Late_For_Dedicated_Environment : exception;
  
   procedure Task_Dedicated_Environment;

   procedure Task_Done (T : Task_ID := Current_Task);
   -- Call it, when you are sure about using the task is done.
   
   -- Should call before first OCI call.
   procedure Set_Events_Support;
   procedure Set_Objects_Support;

   function Handle (ref : Handle_Reference'Class) return System.Address;

   pragma Inline (Handle);
   
private

   type Handle_Reference is abstract 
        new Rf.Controlled_Reference with record
      Handle : OCIHandle := Empty_Handle; 
   end record;

   package AF renames Ada.Finalization;

   type Limited_Variable is new AF.Limited_Controlled with record
      Indicator : aliased Sb2 := Null_Indicator;
      Bind     : aliased OCIBind := OCIBind (Empty_Handle);
      Define  : aliased OCIDefine := OCIDefine (Empty_Handle);
   end record;

   procedure Free (H : in out OCIHandle; HType : Ub4);
   function Alloc_Handle (Parent : OCIEnv; Htype : Ub4) return OCIHandle;
   function Alloc_Descriptor (Parent : OCIEnv; Htype : Ub4) return OCIHandle;
   
   function Get_Attr (Param : OCIHandle; HType : ub4; Attr : Ub4) return Integer;
   function Get_Attr (Param : OCIHandle; HType : ub4; Attr : Ub4) return String;

   procedure Set_Attr (H : OCIHandle; HType : ub4; Value : String; Attr : Ub4);
   
   procedure Set_Attr (H : OCIHandle; HType : ub4; Value : OCIHandle; Attr : Ub4);

end OCI.Thick;